"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2018 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const loader_utils_1 = require("loader-utils");
const sharp_1 = __importDefault(require("sharp"));
const DEFAULT_OPTIONS = {
    deviceWidths: [768, 1080, 1440, 1920, 4096],
    breakpoints: {
        xs: '0px',
        sm: '600px',
        md: '960px',
        lg: '1280px',
        xl: '1920px',
    },
    jpeg: {
        progressive: true,
        trellisQuantisation: true,
        overshootDeringing: true,
        optimizeScans: true,
    },
    png: {
        adaptiveFiltering: true,
    },
    webp: {
        quality: 80,
        alphaQuality: 100,
        lossless: true,
    },
};
function createImage(ctx, options, content, format, width) {
    return __awaiter(this, void 0, void 0, function* () {
        let s = sharp_1.default(content)
            .resize(width)
            .toFormat(format);
        if (format === 'jpg') {
            s = s.jpeg(options.jpeg);
        }
        else if (format === 'png') {
            s = s.png(options.png);
        }
        else if (format === 'webp') {
            s = s.webp(options.webp);
        }
        const converted = yield s.toBuffer();
        const outputPath = loader_utils_1.interpolateName(ctx, `[name].${width}.[hash].${format}`, {
            context: ctx.context,
            content: converted,
        });
        ctx.emitFile(outputPath, converted, undefined);
        // eslint-disable-next-line no-underscore-dangle
        const publicPathDir = ctx._compiler.options.output.publicPath || '';
        const publicPath = `${publicPathDir}${outputPath}`;
        return { format, width, path: publicPath };
    });
}
function run(ctx, content) {
    return __awaiter(this, void 0, void 0, function* () {
        ctx.async();
        ctx.cacheable(true);
        if (!ctx.resourceQuery) {
            // Passthrough when no options specified.
            const outputPath = loader_utils_1.interpolateName(ctx, '[name].[hash].[ext]', {
                content,
                context: ctx.context,
            });
            ctx.emitFile(outputPath, content, undefined);
            const publicPath = `__webpack_public_path__ + ${JSON.stringify(outputPath)}`;
            ctx.callback(undefined, `export default ${publicPath}`);
            return;
        }
        const resourceOptions = loader_utils_1.parseQuery(ctx.resourceQuery);
        const loaderOptions = Object.assign({}, DEFAULT_OPTIONS, loader_utils_1.getOptions(ctx));
        const { lossy } = resourceOptions, breakpoints = __rest(resourceOptions, ["lossy"]);
        if (lossy) {
            loaderOptions.webp.lossless = false;
        }
        const formats = ['webp', lossy ? 'jpg' : 'png'];
        const deviceWidths = loaderOptions.deviceWidths;
        const widths = Object.keys(breakpoints)
            .map((key) => breakpoints[key])
            .map((viewportWidth) => viewportWidth / 100)
            .reduce((values, value) => [
            ...values,
            ...deviceWidths.map((deviceWidth) => Math.ceil(deviceWidth * value)),
        ], []);
        const createImagePromises = [];
        for (const format of formats) {
            for (const width of widths) {
                createImagePromises.push(createImage(ctx, loaderOptions, content, format, width));
            }
        }
        const results = yield Promise.all(createImagePromises);
        const reversedBreakpoints = Object.keys(breakpoints).reverse();
        const sizes = reversedBreakpoints
            .map((breakpoint, i) => i !== reversedBreakpoints.length - 1
            ? `(min-width: ${loaderOptions.breakpoints[breakpoint]}) ${breakpoints[breakpoint]}vw`
            : `${breakpoints[breakpoint]}vw`)
            .join(', ');
        const sources = formats.map((format) => ({
            sizes,
            type: format === 'jpg' ? 'image/jpeg' : `image/${format}`,
            srcSet: results
                .filter((image) => image.format === format)
                .sort((a, b) => b.width - a.width)
                .map((image) => `${image.path} ${image.width}w`)
                .join(', '),
        }));
        const fallback = results
            .filter((image) => image.format !== 'webp')
            .sort((a, b) => a.width - b.width)[0].path;
        const output = {
            sources,
            fallback,
        };
        ctx.callback(undefined, `export default ${JSON.stringify(output)}`);
    });
}
function curioImageLoader(content) {
    run(this, content).catch((e) => this.callback(e));
}
exports.default = curioImageLoader;
exports.raw = true;
//# sourceMappingURL=index.js.map